import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d

################################################################################

def genRotMat(axis,angle):
    """Returns a general rotation matrix;
     when given a normalised axis vector (u)
     and the right handed rotation angle (theta),
      as given in Materials part II C6"""
    m = np.cos(angle)
    n = np.sin(angle)
    u = axis

    rot_mat_pt1 = np.array([[m, u[2]*n, -u[1]*n], [-u[2]*n, m, u[0]*n], [u[1]*n, -u[0]*n, m]])
    rotmat = np.outer(u,u)*(1-m) + rot_mat_pt1

    return rotmat

################################################################################

def plot_chains(chain_list):
    """ Takes a list of 'Polymer_Chain' objects and plots them together on a single canvas"""

    fig = plt.figure()
    ax = plt.axes(projection="3d")

    max_coords = np.zeros(3)
    min_coords = np.zeros(3)

    for chain in chain_list:
        if chain:
            points = chain.coords

            max_coords = np.maximum(np.max(points, axis=0), max_coords)
            min_coords = np.minimum(np.min(points, axis=0), min_coords)

            ax.plot3D(points[:,0],points[:,1],points[:,2], label=chain.name)


    centre = (max_coords + min_coords)/2
    axis_range = np.amax(max_coords - min_coords)*0.55

    ax.set_xlim(centre[0]-axis_range, centre[0]+axis_range)
    ax.set_ylim(centre[1]-axis_range, centre[1]+axis_range)
    ax.set_zlim(centre[2]-axis_range, centre[2]+axis_range)

    ax.set_xlabel("x")
    ax.set_ylabel("y")
    ax.set_zlabel("z")

    fig.legend()
    fig.tight_layout()
    return fig

################################################################################

class Polymer_Chain():
    # initialise counter of all chains
    count = 0

    def __init__(self, chain_length):
        self.name = None
        self._length = chain_length
        self._coords = None
        self.CoM = None
        self.end2end = None
        self.theor_end2end = self.theoretical_end2end()
        self.RoG = None
        self.theor_RoG = self.theoretical_RoG()
        Polymer_Chain.count += 1

    # length defined as a property num. of segments num. # of points !!!
    @property
    def length(self):
        return self._length

    @length.setter
    def length(self, chain_length):
        self._length = chain_length

    # coords defined as a property
    @property
    def coords(self):
        return self._coords

    @coords.setter
    def coords(self, chain_coords):
        self._coords = chain_coords

    # name getter and setter methods
    def get_name(self):
        return self.name

    def set_name(self, chain_name):
        self.name = chain_name

    # describe method
    def describe(self):
        print(self.name)

    # centre of mass calculator
    def calculate_CoM(self):
        if type(self._coords) != "NoneType":
            self.CoM = np.sum(self._coords, axis = 0) / (self.length + 1)
        else:
            print("Need to generate coords with '.calculate_coords()' before trying to find CoM")
        return self.CoM

    # radius of gyration calculator
    def calculate_RoG(self):
        self.calculate_CoM()
        self.RoG = np.sqrt(np.sum((self._coords - self.CoM * np.ones(np.shape(self._coords)))**2) / (self.length + 1))
        return self.RoG

    # end to end distance calculator
    def calculate_end2end(self):
        if type(self._coords) != "NoneType":
            self.end2end = np.sqrt(np.sum((self._coords[0,:] - self._coords[-1,:])**2))
            return self.end2end
        else:
            print("Need to generate coords with '.calculate_coords()' before trying to find end to end distance")

    # theoretical end to end distance calulator: <r> = a * N^0.5
    def theoretical_end2end(self):
        self.theor_end2end = np.sqrt(self.length)
        return self.theor_end2end

    # theoretical radius of gyration calculator: <rg> = <r> / 6^0.5
    def theoretical_RoG(self):
        self.theor_RoG = self.theor_end2end / np.sqrt(6)
        return self.theor_RoG

################################################################################

class Random_Walk_Chain(Polymer_Chain):
    # set segment offset as class variable
    segment_offset = 1

    # initialise count of instances of this class
    count = 0

    # initialise the object, getting the bits from the superclass and incrementing the class instance counter
    def __init__(self, chain_length):
        super().__init__(chain_length)
        Random_Walk_Chain.count += 1
        self.set_name("RW Chain {}".format(Random_Walk_Chain.count))

    def calculate_coords(self):

        number_of_points = self._length + self.segment_offset

        # determine the orientation of each link
        theta = np.random.rand(number_of_points) * np.pi
        phi = np.random.rand(number_of_points) * 2 * np.pi

        # from angles determine each link as a vector
        adVects = np.zeros((number_of_points,3))
        adVects[:,0] = np.sin(theta)*np.cos(phi)
        adVects[:,1] = np.sin(theta)*np.sin(phi)
        adVects[:,2] = np.cos(theta)*np.ones((number_of_points))

        # sum cumulatively over the additional vectors to get each point in the chain
        self._coords = adVects.cumsum(axis = 0)

################################################################################

class Freely_Rotating_Chain(Polymer_Chain):
    # set segment offest as class variable
    segment_offset = 1

    # initialise count of instances of this class
    count = 0

    # initialise the object, getting the bits from the superclass and incrementing the class instance counter
    def __init__(self, chain_length, chain_bond_angle=109.5):
        super().__init__(chain_length)
        self.bond_angle = chain_bond_angle    # chain bond angle preset to carbon value
        Freely_Rotating_Chain.count += 1
        self.set_name("FR Chain {}".format(Freely_Rotating_Chain.count))

    def calculate_coords(self):

        number_of_points = self._length + self.segment_offset

        # set number of points and initialise coordinates, make first link along z axis
        points = np.zeros((number_of_points,3))
        start = np.array([0,0,1])
        points[1,:] += start

        # define the bend angle from the bond angle (fixed for freely rot), and the random torsion angles
        theta = (180 - self.bond_angle) * (np.pi/180)
        phi = np.random.rand(number_of_points) * 2 * np.pi

        # from angles determine additional vectors, one to be added after each rotation
        adVects = np.zeros((number_of_points,3))
        adVects[:,0] = np.sin(theta)*np.cos(phi)
        adVects[:,1] = np.sin(theta)*np.sin(phi)
        adVects[:,2] = np.cos(theta)*np.ones((number_of_points))

        # add the additional vector to the previous one (which lies along the z axis) and then rotate the new vector to the z axis
        row = 0
        for i in range(number_of_points - 2):
            row = i+2

            # determine new chunk to be added and add it
            adVect = adVects[row, :]
            newRow = points[row-1,:] + adVect
            points[row,:] += newRow

            # find rotation axis unit vector (normalised by the rotation angle, which should always be the bond angle)
            u = np.cross(start,adVect) / np.sin(theta)

            # form rotation matrix
            rotmat = genRotMat(u,theta)

            # apply rotation matrix
            points = np.transpose(np.matmul(rotmat,np.transpose(points)))

        self._coords = points

################################################################################

class Rotational_Isomeric_Chain(Polymer_Chain):
    # set segment offest as class variable
    segment_offset = 1

    # initialise count of instances of this class
    count = 0

    # initialise the object, getting the bits from the superclass and incrementing the class instance counter
    def __init__(self, chain_length, chain_bond_angle = 109.5, chain_torsion_positions = 3, chain_trans_bias = 2):
        super().__init__(chain_length)
        self.bond_angle = chain_bond_angle                  # chain bond angle preset to carbon value
        self.torsion_positions = chain_torsion_positions    # chain torsion positions set to 3 as is standard for carbon
        self.trans_bias = chain_trans_bias                  # chain trans bias set to 2, as from part II C10 p16 P(trans) ~= 2*P(gauche) ~= 2*P(gauche-)
        Rotational_Isomeric_Chain.count += 1
        self.set_name("RI Chain {}".format(Rotational_Isomeric_Chain.count))

    def calculate_coords(self):

        number_of_points = self._length + self.segment_offset

        # set number of points and initialise coordinates, make first link along z axis
        points = np.zeros((number_of_points,3))
        z_axis = np.array([0,0,1])
        y_axis = np.array([0,1,0])
        points[1,:] += z_axis


        # define the bend angle, and the randomly selected torsion angles
        phi_options = (np.arange(self.torsion_positions) * 2 * np.pi /self.torsion_positions) + np.pi
        phi_weights = np.ones(self.torsion_positions)
        phi_weights[0] *= self.trans_bias
        phi_weights = phi_weights/np.sum(phi_weights)
        phi = np.random.choice(phi_options, size=number_of_points, p=phi_weights)

        theta = (180 - self.bond_angle) * (np.pi/180)

        # from angles determine additional vectors, one to be added after each rotation
        adVects = np.zeros((number_of_points,3))
        adVects[:,0] = np.sin(theta)*np.cos(phi)
        adVects[:,1] = np.sin(theta)*np.sin(phi)
        adVects[:,2] = np.cos(theta)*np.ones((number_of_points))

        # add the additional vector to the previous one (which lies along the z axis) and then rotate the new vector to the z axis
        row = 0
        for i in range(number_of_points - 2):
            row = i+2

            # determine new chunk to be added and add it
            adVect = adVects[row, :]
            newRow = points[row-1,:] + adVect
            points[row,:] += newRow

            # define matrices so that new segment aligns with z axis and previous one lies in xz plane (these could be vectorised so that they don't have to be done in the for loop, as are't dependent on previous rows)
            rotmatZ = genRotMat(z_axis,phi[row])
            rotmatY = genRotMat(y_axis,theta)

            # apply rotation matrices
            points = np.transpose(np.matmul(rotmatY,np.matmul(rotmatZ,np.transpose(points))))

        self._coords = points

################################################################################



import PySimpleGUI as sg
from . import polymer_chains as pc
import os

# chain options available in the GUI
chain_options = ["Random Walk", "Freely Rotating", "Rotational Isomeric", "None"]
calculator_checkboxes = ["N", "l", "E2E", "CoM", "RoG"]
calculator_checkbox_tooltips = ["Number of segments", "Length of each segment", "End to end distance", "Centre of mass", "Radius of gyration"]
calculator_checkbox_keys = ["-check_{}-".format(label) for label in calculator_checkboxes]

# Build the main layout of the GUI
# set number of chains to create frames for
chain_number = 5
top_row_height = 100
bottom_row_height = 500
left_col_width = 300
right_col_width = 650
element_justification = "center"
pad = (0,0)
canvas_size = (600,400)
calculator_checkboxes_per_line = 5
chains_column_height = 500

# create a chain frame for each chain and put it in the chains column
chain_objects = []
chains_column_layout = []
chain_frame_RC = ["confirm-", "print_coords-"]
chain_combos = []#######################################################################################################################################
for i in range(1, chain_number+1):
    new_chain_frame_layout = [
        [sg.T("Chain type:"), sg.Combo(chain_options, key="-chain{}_type-".format(i), default_value="Random Walk")],
        [sg.T("Chain length:"), sg.Input(key="-chain{}_length-".format(i), enable_events=True, default_text="100", size=(6,1))]
    ]
    new_chain_frame_column = sg.Column(new_chain_frame_layout, pad=pad)
    new_chain_frame = sg.Frame("Chain {}:".format(i), [[new_chain_frame_column]], right_click_menu=["", [("-chain{}_".format(i) + option) for option in chain_frame_RC]])
    chains_column_layout.append([new_chain_frame])
    chain_objects.append(pc.Random_Walk_Chain(100))
chains_column = sg.Column(layout=chains_column_layout, size=(left_col_width-20, chains_column_height), pad=(0,0))

# precalculate chain coords for initial chains
for chain in chain_objects:
    chain.calculate_coords()
    chain.calculate_CoM()
    chain.calculate_RoG()
    chain.calculate_end2end()

# create calulator frame
calculator_frame_layout = []
calculator_frame_layout_line = []
for i, label in enumerate(calculator_checkboxes):
    new_checkbox = sg.Checkbox(label, key=calculator_checkbox_keys[i], tooltip=calculator_checkbox_tooltips[i])
    calculator_frame_layout_line.append(new_checkbox)
    if ((i+1) % calculator_checkboxes_per_line == 0) or i == len(calculator_checkboxes)-1:
        calculator_frame_layout.append(calculator_frame_layout_line)
        calculator_frame_layout_line = []
calculator_frame_layout.append([sg.B("Calculate", key="-calculate_chain_properties-"), sg.B("Save", key="-save_chain_properties-")])
calculator_frame = sg.Frame("Properties Calculator", layout = calculator_frame_layout)

# new layout method
chain_options_frame_column = sg.Column([[calculator_frame], [sg.B("Confirm all", key="-confirm_all_chains-"), sg.B("Clear all", key="-clear_all_chains-")], [chains_column]], size=(left_col_width,bottom_row_height), scrollable=True, vertical_scroll_only=True)
chain_options_frame = sg.Frame("Chain Options", layout=[[chain_options_frame_column]])
plotting_frame_column = sg.Column(layout = [
    [sg.T('Controls:')],
    [sg.Canvas(key='controls_cv', size = (100,30), pad=(0,0))],
    [sg.T('Figure:')],
    [sg.Column(
        layout=[
            [sg.Canvas(key='fig_cv',
                       # it's important that you set this size
                       size=canvas_size
                       )]
        ],
        background_color='#DAE0E6',
        pad=(0, 0)
        )]], size=(right_col_width,bottom_row_height))
plotting_frame = sg.Frame("Plotting", key="-plotting_frame-", layout = [[plotting_frame_column]])
output_multiline = sg.Multiline("This is the text output\n", key="-output_textbox-", size=(80,5), write_only=True, reroute_cprint=True)
output_frame_column = sg.Column([[output_multiline, sg.Column([[sg.B("Popout", key="-output_popout-")], [sg.B("Clear", key="-output_clear-")]])]], size=(right_col_width,top_row_height))
output_frame = sg.Frame("Info", layout = [[output_frame_column]])
main_options_frame_column = sg.Column(layout = [
    [sg.T('Demo')],
    [sg.B('Plot'), sg.B('Exit'), sg.B("Help", key="-help-")]], size=(left_col_width,top_row_height))
main_options_frame = sg.Frame("Main", layout = [[main_options_frame_column]])

# Function to get the help_text file
def get_helptext():
   helptext = """Hello! Welcome to this polymer plotting program.\nThis program plots the shape of polymer chains as formed by some simple models (plus maybe in future some demo shapes).\n\nDISCLAIMER: This program was written by a part II materials student with < 1 year coding experience.\nSome bits may not be best optimised, pls no hate, feedback always welcome :)\n\nPlease read this help text before sending any issues.\nFor code issues: https://github.com/cnswoolley10/Polymer-Plotting\n\n-----------------------------------------------------------------------------------------------\nUSING THIS GUI:      (You don't have to use the GUI, see the GitHub readme above)\n\n1. Set the values for the chains in the left hand pain. Keep total sum of chain length<20000 for fast calculation.\n2. Check your chain values and then click confirm all. This will calculate your chain geometries and store them in memory.\n3. Click plot to see your chains!\n-----------------------------------------------------------------------------------------------\nFULL INFO:\nThere are four panes:\nMAIN:\n\t- Plot - Plots the chains stored in the memory\n\t- Exit - Exits the GUI and ends the program\n\t- Help - Looks like you've figured this one out\n\nCHAIN OPTIONS:\n\t- Confirm all - Takes the chains as specified below this button, calculates their shapes and saves them to memory. This is done rather than to auto-calculate because the program begins to slow for chains longer than ~10000 segments\n\t- Clear all\t  - NOT YET IMPLEMENTED Sets all the chains to 'None' and saves to memory\n\t\n\tCHAIN X:\n\t\t- Chain type   - Set the model with which the chain is created\n\t\t- Chain length - Set the number of segments in the chain\n\t\tNOTE: You'll see most parameters of the models are currently only presets and not user-editable (this will change in future)\n\t\t\tThe current presets are: (Chosen to reflect carbon chain properties)\n\t\t\t\tALL: segment length = 1\n\t\t\t\tFR & RI: bond angle = 109.5\n\t\t\t\tRI: number of torsion positions = 3 (t, g+, g-)\n\t\t\t\t\ttrans configuration bias = 2 (how many times more likely the trans configuration is compared to any one of the others)\n\t\t- Right click  - Right click the chain name to confirm chains individually or print the stored coordinates\n\t\n\tPROPERTIES CALCULATOR:\n\t\tProperties are:\n\t\t\t- N\t    - Number of segments in the chains\n\t\t\t- l     - Length of each segments\n\t\t\t- E2E   - End to end distance\n\t\t\t- CoM   - Centre of Mass\n\t\t\t- RoG   - Radius of gyration\n\t\t- Calculate - Calculates the selected properties of the chains stored in the memory and prints to info pane\n\t\t- Save      - Save to .csv the selected properties of the chains stored in the memory\n\nPLOTTING:\n\tThis is all the standard GUI imported from matplotlib.\n\tNOTE: the zoom button doesn't work due to being in 3D.\n\t\tCan still zoom by click&drag with right mouse button\n\t\tLMB to pan as you'd expect.\n\t\tMMB to traverse.\n\nINFO:\n\tAny calculations will be printed here as well as info messages.\n\tGet a larger version of the textbox with the popout button. You can edit and save the output text from the popout.\n"""
   return helptext

# This is to include a matplotlib figure in a Tkinter canvas
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
def draw_figure_w_toolbar(canvas, fig, canvas_toolbar):
    if canvas.children:
        for child in canvas.winfo_children():
            child.destroy()
    if canvas_toolbar.children:
        for child in canvas_toolbar.winfo_children():
            child.destroy()
    figure_canvas_agg = FigureCanvasTkAgg(fig, master=canvas)
    figure_canvas_agg.draw()
    toolbar = Toolbar(figure_canvas_agg, canvas_toolbar)
    toolbar.update()
    figure_canvas_agg.get_tk_widget().pack(side='right', fill='both', expand=1)
class Toolbar(NavigationToolbar2Tk):
    def __init__(self, *args, **kwargs):
        super(Toolbar, self).__init__(*args, **kwargs)

# Function to open a second window to display text
def open_text_window(text, key, size=(100,30), title="Output Text"):
    layout = [[sg.Multiline(text, key=key, size=size, write_only=True)], [sg.B("Save All", key="-save_all_output-")]]
    window = sg.Window(layout=layout, title=title, modal=True)
    choice = None
    while True:
        event, values = window.read()
        print(event)
        if event == "Exit" or event == sg.WIN_CLOSED:
            break
        elif event == "-save_all_output-":
            save_location = sg.popup_get_file("Select a save location", save_as=True, initial_folder=os.getcwd(), default_extension=".txt")
            pass


    window.close()

# Function to output the selected chain properties to the output box
def calculate_chain_properties(chain_objects, values, cprint=True, round=True):
    keys_to_output = ["Chain_name"]
    for key in calculator_checkbox_keys:
        if values[key]:
            keys_to_output.append(key)
    output_properties_lines = []
    output_properties_header = ""
    for key in keys_to_output:
        if key in calculator_checkbox_keys:
            if key == calculator_checkbox_keys[3]:
                output_properties_label = calculator_checkboxes[calculator_checkbox_keys.index(key)]
                output_properties_header += "{}_x,{}_y,{}_z,".format(output_properties_label,output_properties_label,output_properties_label)
            else:
                output_properties_header += calculator_checkboxes[calculator_checkbox_keys.index(key)] + ","
        else:
            output_properties_header += key + ","
    output_properties_lines.append(output_properties_header)
    for chain in chain_objects:
        output_properties_chain_line = ""
        if chain:
            if round:
                for key in keys_to_output:
                    if key == "Chain_name":
                        output_properties_chain_line += chain.name + ","
                    elif key == calculator_checkbox_keys[0]:
                        output_properties_chain_line += "{:.2e},".format(chain.length)
                    elif key == calculator_checkbox_keys[1]:
                        output_properties_chain_line += "{},".format(1)
                    elif key == calculator_checkbox_keys[2]:
                        output_properties_chain_line += "{:.2e},".format(chain.end2end)
                    elif key == calculator_checkbox_keys[3]:
                        output_properties_chain_line += "{:.2e},{:.2e},{:.2e},".format(chain.CoM[0],chain.CoM[1],chain.CoM[2])
                    elif key == calculator_checkbox_keys[4]:
                        output_properties_chain_line += "{:.2e},".format(chain.RoG)
                output_properties_lines.append(output_properties_chain_line)
            else:
                for key in keys_to_output:
                    if key == "Chain_name":
                        output_properties_chain_line += chain.name + ","
                    elif key == calculator_checkbox_keys[0]:
                        output_properties_chain_line += "{},".format(chain.length)
                    elif key == calculator_checkbox_keys[1]:
                        output_properties_chain_line += "{},".format(1)
                    elif key == calculator_checkbox_keys[2]:
                        output_properties_chain_line += "{},".format(chain.end2end)
                    elif key == calculator_checkbox_keys[3]:
                        output_properties_chain_line += "{},{},{},".format(chain.CoM[0],chain.CoM[1],chain.CoM[2])
                    elif key == calculator_checkbox_keys[4]:
                        output_properties_chain_line += "{},".format(chain.RoG)
                output_properties_lines.append(output_properties_chain_line)
    if cprint:
        sg.cprint("\nCalculated Properties:")
        for line in output_properties_lines:
            sg.cprint(line)
    return output_properties_lines

# Function to save the calculated chain properties
def save_chain_properties(chain_objects, values):
            chain_properties_to_save = calculate_chain_properties(chain_objects, values, cprint=False, round=False)
            chain_properties_to_save = [line + "\n" for line in chain_properties_to_save]
            #print(chain_properties_to_save)
            save_location = sg.popup_get_file("Select a save location", save_as=True, initial_folder=os.getcwd(), default_extension=".csv", file_types=(("Comma separated values", ".csv"),))
            if save_location:
                with open(save_location,"w") as save_properties_file:
                    save_properties_file.writelines(chain_properties_to_save)

# Function for confirm chain button
def confirm_chain(chain_objects, chain_index, values):
    chain_int = chain_index + 1
    calculate_coords_ = True
    if values["-chain{}_type-".format(chain_int)] == chain_options[0]:
        chain_objects[chain_index] = pc.Random_Walk_Chain(int(values["-chain{}_length-".format(chain_int)]))
    elif values["-chain{}_type-".format(chain_int)] == chain_options[1]:
        chain_objects[chain_index] = pc.Freely_Rotating_Chain(int(values["-chain{}_length-".format(chain_int)]))
    elif values["-chain{}_type-".format(chain_int)] == chain_options[2]:
        chain_objects[chain_index] = pc.Rotational_Isomeric_Chain(int(values["-chain{}_length-".format(chain_int)]))
    else:
        chain_objects[chain_index] = None
        calculate_coords_ = False
    if calculate_coords_:
        chain_objects[chain_index].calculate_coords()
        chain_objects[chain_index].calculate_CoM()
        chain_objects[chain_index].calculate_RoG()
        chain_objects[chain_index].calculate_end2end()

# Function for clear chain button
def clear_chain(chain_objects, chain_index): ###########################################################################################################################################
    chain_objects[chain_index] = None
    sg.update(value)
# Function to get the index of a chain when provided an event string
def get_chain_index(event):
    chain_index_start = event.index("-chain") + 6
    chain_index_end = event.index("_")
    chain_index = int(event[chain_index_start:chain_index_end]) -1
    return chain_index



import PySimpleGUI as sg
from . import polymer_chains as pc
from . import GUI_tools as GUI

def main():
    # ------------------------------- PySimpleGUI CODE
    # define chain options for dropdown
    chain_options = GUI.chain_options

    # get help_text
    help_text = GUI.get_helptext()

    layout = [
        [GUI.main_options_frame, GUI.output_frame],
        [GUI.chain_options_frame, GUI.plotting_frame]]

    output_multiline=GUI.output_multiline
    canvas_size = GUI.canvas_size
    chain_objects = GUI.chain_objects
    window = sg.Window('Polymer Plotting', layout, resizable=True)

    # Event loop for main window
    while True:
        event, values = window.read()
        print(event)
        # close loop if window closed or exit button pressed
        if event in (sg.WIN_CLOSED, 'Exit'):  # always,  always give a way out!
            break
        # plot chains if plot button is pressed
        if event == 'Plot':
            fig = pc.plot_chains(chain_objects)
            DPI = fig.get_dpi()
            fig.set_size_inches(canvas_size[0] / float(DPI), canvas_size[1] / float(DPI))
            GUI.draw_figure_w_toolbar(window['fig_cv'].TKCanvas, fig, window['controls_cv'].TKCanvas)
        # prevent input values being anything but integers
        if event[-8:] == "_length-" and values[event] and values[event][-1] not in ('0123456789'):
            window[event].update(values[event][:-1])
        # RC (Right Click): calculate coords and other chain values for chain object once it is confirmed
        if event[-9:] == "_confirm-":
            chain_index = GUI.get_chain_index(event)
            GUI.confirm_chain(chain_objects, chain_index, values)
        # RC: print the coords of the chain to the output box
        if event[-13:] == "print_coords-":
            chain_index = GUI.get_chain_index(event)
            sg.cprint("\n")
            sg.cprint(chain_objects[chain_index].coords)
        # confirm all the chains at once
        if event == "-confirm_all_chains-":
            for chain_index in range(len(chain_objects)):
                GUI.confirm_chain(chain_objects, chain_index, values)
        # clear all the chains at once
        if event == "-clear_all_chains-":##############################################################################################################
            sg.cprint("The button works!")
        # create a help window
        if event == "-help-":
            GUI.open_text_window(help_text,"-help_window-", title="Help Text", size=(150,30))
        # create a popout window version of the output box
        if event == "-output_popout-":
            GUI.open_text_window(GUI.output_multiline.get(), "Expanded Output Textbox", title="Expanded Output Textbox")
        # clear the output box
        if event == "-output_clear-":
            output_multiline.update(value="")
        # print the selected properties of all the chains to the output box
        if event == "-calculate_chain_properties-":
            GUI.calculate_chain_properties(chain_objects, values)
        # save the current properties to a .csv file
        if event == "-save_chain_properties-":
            GUI.save_chain_properties(chain_objects, values)
    window.close()
